/*
 * Copyright (C) 2013
 * 
 * Stefan Fafianie
 *
 * This package is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This package is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * This package makes use of a selection of classes from the libtw library 
 * which is also published under the GNU Lesser General Public license.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this package; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package order;

import java.util.BitSet;
import libtw.input.GraphInput.InputData;

/* Forget vertex node with two children.
 * We compare the partial solutions returned by both children
 * and return the smallest. If a partial solution from one child
 * is returned * then we request a new one from this child. */
public class ForgetVertexNode<D extends InputData> extends Node<D> {

	Node<D> childInclude, childExclude;
	int vertexID;
	PartialSolution include, exclude;
	boolean askInclude, askExclude, specialCase, cleanupInclude, cleanupExclude;
	int checkSingleton, phase;
	
	public ForgetVertexNode (BitSet subset, int vertexID, OrderDP<D> dp){
		super(subset, dp);
		this.subset = subset;
		this.vertexID = vertexID;
		askInclude = true;
		askExclude = true;
		cleanupInclude = false;
		cleanupExclude = false;
		
	}
	
	public void init(){
		int total = 0; 
		int position = 0;
		for (int i = 0; i < childInclude.subset.size(); i++){
			if (childInclude.subset.get(i)){
				total++;
				if (i < vertexID){
					position++;
				}
			}
		}
		
		int size = (int)Math.pow(2, total-1);
		
		if (position == 0){
			specialCase = true;
			checkSingleton = size-1;
			
		}
		else{
			specialCase = false;
			checkSingleton = (int)Math.pow(2, total-position-1);
			phase = size/(int) Math.pow(2, position-1);
		}
	}
	
	public PartialSolution applyNext() {
		
		boolean passInclude;
		
		if (askInclude){
			include = childInclude.reduceNext();
			if (include == null){
				cleanupInclude = true;
			}
			askInclude = false;
		}
		if (askExclude){
			exclude = childExclude.reduceNext();
			if (exclude == null){
				cleanupExclude = true;
			}
			askExclude = false;
		}
		
		if (cleanupInclude && cleanupExclude){
			hasNext = false;
			return null;
		}
		
		if (cleanupInclude){
			passInclude = false;
		}
		else if (cleanupExclude){
			passInclude = true;
		}
		else if (include.cost < exclude.cost){
			passInclude = true;
		}
		else{
			passInclude = false;
		}
		
		if (passInclude){
			if (!childInclude.hasNext){
				cleanupInclude = true;
			}
			else{
				askInclude = true;
			}
			if (include.row.get(checkSingleton)){
				return applyNext();
			}
			else{
				dp.operations++;
				return include.forgetVertex(specialCase, phase);
			}
			
		}
		else{
			if (!childInclude.hasNext){
				cleanupExclude = true;
			}
			else{
				askExclude = true;
			}
			return exclude;
		}
	}

	@Override
	public String toString() {
		return "forget " + vertexID;
	}

}
