/*
 * Copyright (C) 2013
 * 
 * Stefan Fafianie
 *
 * This package is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This package is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * This package makes use of a selection of classes from the libtw library 
 * which is also published under the GNU Lesser General Public license.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this package; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package order;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.List;
import libtw.input.GraphInput.InputData;

/* Introduce vertex node in which the new vertex is not introduced. 
 * This node has a sibling that does introduce the new vertex.
 * If this node or the sibling needs a new partial solution we must
 * make sure that both nodes have access to it. 
 * Partial solutions are returned in the order in which we received them */
public class IgnoreVertexNode<D extends InputData> extends Node<D> {

	Node<D> childNode, siblingIntroduce;
	int vertexID;
	boolean terminal;
	boolean cleanup;
	List<PartialSolution> solutions;
	
	public IgnoreVertexNode(BitSet subset, int vertexID, boolean terminal, OrderDP<D> dp){
		super(subset, dp);
		this.checkUnique = false;
		this.subset = subset;
		this.vertexID = vertexID;
		this.terminal = terminal;
		if (terminal){
			hasNext = false;
		}
		solutions = new ArrayList<PartialSolution>();
		cleanup = false;
		reduce = false;
	}

	public PartialSolution applyNext() {
		if(terminal){ 
			return null;
		}
		
		if (solutions.isEmpty()){
			if (cleanup){
				hasNext = false;
				return null;
			}
			else{
				getNext();
				return applyNext();
			}
		}
		else{
			return solutions.remove(0);
		}
	}
	
	void getNext(){
		PartialSolution next = childNode.reduceNext();
		if (next == null){
			cleanup = true;
		}
		else{
			solutions.add(next);
			((IntroduceVertexNode<D>) siblingIntroduce).solutions.add(next);
		}
		if (!childNode.hasNext){
			cleanup = true;
		}
	}
	@Override
	public String toString() {
		return "ignore " + vertexID + " " + terminal; 
	}
	
	

}
