/*
 * Copyright (C) 2013
 * 
 * Stefan Fafianie
 *
 * This package is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This package is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * This package makes use of a selection of classes from the libtw library 
 * which is also published under the GNU Lesser General Public license.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this package; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package order;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.List;

import libtw.input.GraphInput.InputData;

/* Introduce vertex node in which the new vertex is introduced. 
 * This node has a sibling that does not introduce the new vertex.
 * If this node or the sibling needs a new partial solution we must
 * make sure that both nodes have access to it. 
 * Partial solutions are returned in the order in which we received them */
public class IntroduceVertexNode<D extends InputData> extends Node<D> {

	Node<D> siblingIgnore;
	int vertexID;
	List<PartialSolution> solutions;
	boolean specialCase;
	int phase;
	
	public IntroduceVertexNode(BitSet subset, int vertexID, OrderDP<D> dp){
		super(subset, dp);
		this.subset = subset;
		this.vertexID = vertexID;
		solutions = new ArrayList<PartialSolution>();
		reduce = false;
		checkUnique = false;
		
		int position = 0, total = 0;
		
		for (int i = 0; i < subset.size(); i++){
			if (subset.get(i)){
				total++;
				if (i < vertexID){
					position++;
				}
			}
		}
		
		if (position == 0){
			specialCase = true;
		}
		else{
			specialCase = false;
		}
		
		phase = (int) Math.pow(2, total-position);
		
		
	}
	
	public PartialSolution applyNext() {
		if (solutions.isEmpty()){
			if (((IgnoreVertexNode<D>) siblingIgnore).cleanup){
				hasNext = false;
				return null;
			}
			else{
				((IgnoreVertexNode<D>) siblingIgnore).getNext();
				return applyNext();
			}
		}
		else{
			dp.operations++;
			return solutions.remove(0).IntroduceVertex(specialCase, phase);
		}
	}

	@Override
	public String toString() {
		return "intro " + vertexID;
	}

}
