/*
 * Copyright (C) 2013
 * 
 * Stefan Fafianie
 *
 * This package is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This package is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * This package makes use of a selection of classes from the libtw library 
 * which is also published under the GNU Lesser General Public license.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this package; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package order;

import java.util.BitSet;

/* Similar to Row, i.e. we use the weighted bit string representation
 * The parts of the operations that we need to perform only once for partial solutions
 * corresponding to a fixed subset of vertices have been moved to the nodes.
 */
public class PartialSolution {

	BitSet row;
	public int size, cost;
	
	public PartialSolution(int size){
		row = new BitSet(size);
	    this.size = size;
	}
	
	public static PartialSolution leafSolution(){
		PartialSolution result = new PartialSolution(0);
		result.cost = 0;
		return result;
	}
	
	public PartialSolution forgetVertex(boolean specialCase, int phase) {
		
		PartialSolution result = new PartialSolution(size/2);
		result.cost = cost;
					
		//special case
		if (specialCase){
			for (int i = 0; i < size/2; i++){
				if (row.get(i) || row.get(size-i-1)){
					result.row.set(i);
				}
			}
					return result;
		}
		//maintain pattern
		else{
			int index = 0;
			for (int i = 0; i < size/phase; i++){
				for (int j = 0; j < phase/2; j++){
					if(row.get(i*phase+j) || row.get(i*phase+j+phase/2)){
						result.row.set(index);
					}
					index++;
				}
			}	
			return result;
		}
	
	}

	public PartialSolution IntroduceVertex(boolean specialCase, int phase) {
		
		PartialSolution result;
		
		// extending leaf
		if (size == 0){
			result = new PartialSolution(1);
			result.row.set(0);
			result.cost = 0;
			return result;
		}
			
		result = new PartialSolution(size*2);
		result.cost = cost;
		
		//special case
		if (specialCase){
			for (int i = 0; i < size; i++){
				if (row.get(i)){
					result.row.set(i);
					result.row.set(result.size-i-1);
				}
			}
		}
		//maintain pattern
		else{	
			for (int ind = 0; ind < result.size/2; ind++){
				int j = ind % (phase/2);
				int i = ind / (phase/2);
				if (row.get(ind)){
					result.row.set(i*phase+j);
					result.row.set(i*phase+j+phase/2);
				}				
			}
		}
		
		return result;
	}
	
	public static PartialSolution edgePrint(BitSet verticeSet, int vertexIDA, int vertexIDB, int edgeWeight){
		
		int total = 0, positionA = 0, positionB = 0;
		for (int i = 0; i < verticeSet.size(); i++){
			if (verticeSet.get(i)){
				total++;
				if (i < vertexIDA){
					positionA++;
				}
				if (i < vertexIDB){
					positionB++;
				}
			}
		}
		
		PartialSolution result = new PartialSolution((int) Math.pow(2, total-1));
		int phaseA = result.size*2 / (int) Math.pow(2, positionA);
		int phaseB = result.size*2 / (int) Math.pow(2, positionB);
		for (int i = 0; i < result.size; i++){
			boolean leftA = (i%phaseA) < (phaseA/2);
			boolean leftB = (i%phaseB) < (phaseB/2);
			if ((leftA && leftB) || (!leftA && !leftB)){
				result.row.set(i);
			}
		}
		result.cost = edgeWeight;
		return result;
	}
	
	public PartialSolution joinPartialSolution(PartialSolution otherRow){
		PartialSolution result = new PartialSolution(size);
		for (int i = 0; i < size; i++){
			if (row.get(i) && otherRow.row.get(i)){
				result.row.set(i);
			}
		}
		result.cost = cost + otherRow.cost;
		return result;
	}
	
	public String toString(){
		String result = "";
		for (int i = 0; i < size; i++){
			if (row.get(i)){
				result += "1";
			}
			else{
				result += "0";
			}
		}
		return result;
	}
}
