/*
 * Copyright (C) 2013
 * 
 * Stefan Fafianie
 *
 * This package is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This package is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * This package makes use of a selection of classes from the libtw library 
 * which is also published under the GNU Lesser General Public license.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this package; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package steiner;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;

import libtw.input.DgfReader;
import libtw.input.InputException;
import libtw.input.GraphInput.InputData;
import libtw.ngraph.NEdge;
import libtw.ngraph.NGraph;
import libtw.ngraph.NVertex;

/* Use this class to convert graphs from .dgf format to steiner-tree problem instances.
 * Selects ~20% of vertices as terminal and adds random weights to edges in the interval
 * [1..1000]. Any .dgf file in the given path will be converted to .stp (original files 
 * won't be lost though). */

public class DgfToStp {

	public static void main(String[] args) throws IOException, InputException {
		DgfReader in;
		NGraph<InputData> g = null;
		FileWriter writer;
		File dir = new File(args[0]);
		for (File file: dir.listFiles()){
				if (!file.isDirectory()){
				in = new DgfReader(args[0]+file.getName());
				g = in.get();			
				writer = new FileWriter(file.getName().substring(0, file.getName().length()-4) + ".stp");
				writer.append("33D32945 STP File, STP Format Version 1.0");
				writer.append("\n");
				writer.append("\n");
				writer.append("SECTION Graph");
				writer.append("\n");
				writer.append("Nodes " + g.getNumberOfVertices());
				writer.append("\n");
				writer.append("Edges " + g.getNumberOfEdges());
				writer.append("\n");
				
				for ( Iterator<NEdge<InputData>> iterator = g.edges().iterator(); iterator.hasNext();){
					NEdge<InputData> edge = iterator.next();
					writer.append("E " + edge.a.data.name + " " + edge.b.data.name + " " + ((int)(Math.random()*1000)));
					writer.append("\n");
				}
				writer.append("END");
				writer.append("\n");
				writer.append("\n");
				
				writer.append("SECTION Terminals");
				writer.append("\n");
				
				int terminals = (int)(Math.floor(g.getNumberOfVertices()*0.2));
				
				writer.append("Terminals " + terminals);
				writer.append("\n");
				
				ArrayList<NVertex<InputData>> vertices = new ArrayList<NVertex<InputData>>();
				for ( Iterator<NVertex<InputData>> iterator = g.getVertices(); iterator.hasNext();){
					vertices.add(iterator.next());
				}
				Collections.shuffle(vertices);
				for (int i = 0; i < terminals; i++){
					writer.append("T " + vertices.get(i).data.name);
					writer.append("\n");
				}
		
				writer.append("END");
				writer.append("\n");
				writer.append("\n");
				writer.append("EOF");
				
				writer.flush();
				writer.close();
				}
		}
	}

}
